#include "pix_colortrack.h"

CPPEXTERN_NEW(pix_colortrack)

/////////////////////////////////////////////////////////
//
// pix_colortrack
// This is a GPL license object by Jaime Oliver:
// jaime.oliver@gmail.com / www.jaimeoliver.pe
// ##############################################################################################
// Constructor
// ##############################################################################################
pix_colortrack :: pix_colortrack() {
	inlet_new(this->x_obj, &this->x_obj->ob_pd, gensym("list"),  gensym("coord"));
    inlet_new(this->x_obj, &this->x_obj->ob_pd, gensym("list"),  gensym("redplane"));
	inlet_new(this->x_obj, &this->x_obj->ob_pd, gensym("list"),  gensym("bluplane"));
	inlet_new(this->x_obj, &this->x_obj->ob_pd, gensym("list"),  gensym("bounds"));
	outlet1 = outlet_new(this->x_obj, gensym("list"));
	outlet2 = outlet_new(this->x_obj, gensym("list"));
	outlet3 = outlet_new(this->x_obj, gensym("list"));
	mode = 0;
	histhresh = 0;
	left	= 0;
	top		= 0;
	right	= 319;
	bottom	= 239;
}
// ##############################################################################################
// Destructor
// ##############################################################################################
pix_colortrack :: ~pix_colortrack()
{}

// ##############################################################################################
// Process image function
// ##############################################################################################

void pix_colortrack :: processRGBAImage(imageStruct &image)
{
  unsigned char *pixels = image.data;
  int x2, y2, xcoord, ycoord, xcount, ycount;
  float c2f;
  float rrnull, rgnull, rbnull, brnull, bgnull, bbnull, rtest, btest;
  int i, j, n, totalyesr, totalyesb, totalyesy, totalyesxb, totalyesyb, xval, yval, xval2, yval2, m1x, m1y, m2x, m2y;
  int yesx[image.xsize], yesy[image.ysize], yesx2[image.xsize], yesy2[image.ysize];
  int yesxb[image.xsize], yesyb[image.ysize], yesxb2[image.xsize], yesyb2[image.ysize];



  if (left < 0) left = 0;
  if (top < 0) top = 0;
  if (right > image.xsize) right = image.xsize;
  if (bottom > image.ysize) bottom = image.ysize;
  
	totalyesr = totalyesb = 0;
//	totalyesx = totalyesy = totalyesxb, totalyesyb = 0;
	i = j = xval = yval = yval2 = xval2 = 0;

	m1x = m1y = m2x = m2y = 0;	
	t_atom at[3];
	t_atom ap[4];
	t_atom ab[4];
	c2f = 1./255.; // char to float constant

// ###########################################################################
// mode0 outputs the color of the pixel in the specified x y position
// ###########################################################################

if(mode==0) {  
  if (xpix < 5) x2 = 5; else if (xpix >= image.xsize - 5) x2 = image.xsize - 5; else x2 = xpix;
  if (ypix < 5) y2 = 5; else if (ypix >= image.ysize - 5) y2 = image.ysize - 5; else y2 = ypix; 
  
  SETFLOAT(&at[0], pixels[chRed   + image.csize * x2 + image.csize*image.xsize*y2]*c2f);
  SETFLOAT(&at[1], pixels[chGreen + image.csize * x2 + image.csize*image.xsize*y2]*c2f);
  SETFLOAT(&at[2], pixels[chBlue  + image.csize * x2 + image.csize*image.xsize*y2]*c2f);
  outlet_list(outlet1, 0, 3, at); 
  
  for (ycount = y2 - 3; ycount < y2 + 4; ycount++){
	 ycoord = image.csize * image.xsize * ycount; 		
     if (ycount == y2)
		for (xcount = x2 - 3; xcount < x2 + 4; xcount++){
		xcoord = image.csize * xcount + ycoord; 	
		pixels[chRed   + xcoord] = 0; pixels[chGreen + xcoord] = 255;
		pixels[chBlue  + xcoord] = 0; pixels[chAlpha + xcoord] = 255; }
	else{
		xcount = x2;
		xcoord = image.csize * xcount + ycoord;
		pixels[chRed   + xcoord] = 0; pixels[chGreen + xcoord] = 255; 
		pixels[chBlue  + xcoord] = 0; pixels[chAlpha + xcoord] = 255; }
  }
}	
// ###########################################################################
// mode1 only lets through values above the redplane
// ###########################################################################
else if(mode==1)
{  
/////////////////// clear all histograms
	for (n = 0; n <= image.xsize; n++) { 
		yesx[n]  = 0; yesx2[n] = 0;
		yesy[n]  = 0; yesy2[n] = 0;
		} 
	for (n = image.xsize + 1; n <= image.ysize; n++) { 
		yesy[n]  = 0; yesy2[n] = 0;
		}
/////////////////// find the nullspace for the redplane coordinates
	rrnull = -1 / (1 - rr); 
	rgnull =  1 / rg;
	rbnull =  1 / rb;
	
	post("rrnull = %f, rgnull = %f, rbnull = %f", rrnull, rgnull, rbnull);
		
/////////////////// here is the raster
		for (ycount = top; ycount < bottom; ycount++){
				ycoord = image.csize * image.xsize * ycount;
			for (xcount = left; xcount < right; xcount++){
					xcoord = image.csize * xcount + ycoord;	
			
		rtest = (((pixels[chRed + xcoord]*c2f) - rr) * rrnull) + (pixels[chGreen + xcoord]*c2f * rgnull) + (pixels[chBlue + xcoord]*c2f * rbnull);
			
				if  (rtest <= 0){
					yesx[xcount] += 1; // make a histogram of each axis
					yesy[ycount] += 1;
					totalyesr++;
					pixels[chRed   + xcoord] = 0;
					pixels[chGreen + xcoord] = 255;
					pixels[chBlue  + xcoord] = 0;
					pixels[chAlpha + xcoord] = 255;
					}
			}
		}

		for (i = 0; xval <= (totalyesr * 0.5 ); i++) xval += yesx[i]; //find the median for red
		for (j = 0; yval <= (totalyesr * 0.5 ); j++) yval += yesy[j];
							
		SETFLOAT(&ap[0], i);
		SETFLOAT(&ap[1], j);
		SETFLOAT(&ap[2], totalyesr);
//		SETFLOAT(&ap[3], totalyesry);
		outlet_list(outlet2, 0, 3, ap); 
} 
// ###########################################################################
// mode2 lets through values above the red plane and blue plane
// ###########################################################################
else if (mode == 2)
{
////////////////////////////////////////////// Clear all histograms
	for (n = 0; n <= image.xsize; n++) { 
		yesx[n]   = 0; yesxb[n]  = 0; 
		yesy[n]   = 0; yesyb[n]  = 0; 
		} 
	for (n = image.xsize + 1; n <= image.ysize; n++) { 
		yesy[n]   = 0; 
		yesyb[n]  = 0; 
		}
/////////////////// find the nullspace for the redplane and bluplane coordinates
	rrnull = -1 / (1 - rr); 
	rgnull =  1 / rg;
	rbnull =  1 / rb;
	brnull =  1 / br; 
	bgnull =  1 / bg;
	bbnull = -1 / (1 - bb);
	
////////////////////////////////////////////// Raster
		for (ycount = top; ycount < bottom; ycount++){
				ycoord = image.csize * image.xsize * ycount;
			for (xcount = left; xcount < right; xcount++){
					xcoord = image.csize * xcount + ycoord;	
////////////////////////////////////////////// find out if the incoming pixel is above or below the nullplane
		rtest = ((pixels[chRed + xcoord]*c2f - rr) * rrnull) + ((pixels[chGreen + xcoord]*c2f) * rgnull) + (( pixels[chBlue + xcoord]*c2f      ) * rbnull);	
		btest = ((pixels[chRed + xcoord]*c2f     ) * brnull) + ((pixels[chGreen + xcoord]*c2f) * bgnull) + (((pixels[chBlue + xcoord]*c2f) - bb) * bbnull);	
			
		
				if  (rtest <= 0){
					yesx[xcount]  += 1;					// make a histogram of each axis
					yesy[ycount]  += 1;
					totalyesr++;
					pixels[chRed   + xcoord] = 0  ;
					pixels[chGreen + xcoord] = 255; 
					pixels[chBlue  + xcoord] = 0  ;
					pixels[chAlpha + xcoord] = 255;		// make it green to see it.
					};
				if  (btest <= 0){
					yesxb[xcount] += 1;
					yesyb[ycount] += 1;
					totalyesb++;
					pixels[chRed   + xcoord] = 255;
					pixels[chGreen + xcoord] = 255; 
					pixels[chBlue  + xcoord] = 0  ;
					pixels[chAlpha + xcoord] = 255;		// make it yellow to see it.
					}
			}
		}
////////////////////////////////////////////// Find the median
		for (i = 0                ; i <= image.xsize; i++) {
			xval  +=  yesx[i];  if (xval  <= (totalyesr * 0.5 )) m1x = i;
			xval2 += yesxb[i];	if (xval2 <= (totalyesb * 0.5 )) m2x = i;
			yval  +=  yesy[i];	if (yval  <= (totalyesr * 0.5 )) m1y = i;
			yval2 += yesyb[i];	if (yval2 <= (totalyesb * 0.5 )) m2y = i;
			}
		for (j = (image.xsize + 1); j <= image.ysize; j++) {
			yval  +=  yesy[j];  if (yval  <= (totalyesr * 0.5 )) m1y = j;
			yval2 += yesyb[j];  if (yval2 <= (totalyesb * 0.5 )) m2y = j;
			}
		SETFLOAT(&ap[0], m1x);
		SETFLOAT(&ap[1], m1y);
		SETFLOAT(&ap[2], totalyesr );
	//	SETFLOAT(&ap[3], totalyesy );
		outlet_list(outlet2, 0, 3, ap); 
							
		SETFLOAT(&ab[0], m2x);
		SETFLOAT(&ab[1], m2y);
		SETFLOAT(&ab[2], totalyesb);
//		SETFLOAT(&ab[3], totalyesyb);
		outlet_list(outlet3, 0, 3, ab); 
	} 
}
void pix_colortrack :: processYUVImage(imageStruct &image){
   error("you fool, this aint RGBA");   }
void pix_colortrack :: processGrayImage(imageStruct &image){
   error("you fool, this aint RGBA");   }

// ##############################################################################################
// vecBoundsMess ( Region of Interest )
// ##############################################################################################

void pix_colortrack :: vecBoundsMess(int argc, t_atom *argv){
	top		= (int)atom_getfloat(&argv[0]); // top
	bottom	= (int)atom_getfloat(&argv[1]); // bottom
	left	= (int)atom_getfloat(&argv[2]); // Left
	right	= (int)atom_getfloat(&argv[3]); // Right
	if (top > bottom) bottom = top + 1; 
	if (left > right) right = left + 1;
	post("ROI = %d, b = %d, l = %d, r = %d", top, bottom, left, right);
	setPixModified();}   
// ##############################################################################################
// vecbluplaneMess
// ##############################################################################################

void pix_colortrack :: vecbluplaneMess(int argc, t_atom *argv){
	br		  = (float)atom_getfloat(&argv[0]);
	bg		  = (float)atom_getfloat(&argv[1]);
	bb		  = (float)atom_getfloat(&argv[2]);
	post("bluplane: %f %f %f", br, bg, bb);	
  setPixModified();}
// ##############################################################################################
// vecredplaneMess
// ##############################################################################################

void pix_colortrack :: vecredplaneMess(int argc, t_atom *argv){
	rr		  = (float)atom_getfloat(&argv[0]);
	rg		  = (float)atom_getfloat(&argv[1]);
	rb		  = (float)atom_getfloat(&argv[2]);
	post("redplane: %f %f %f", rr, rg, rb);	
  setPixModified();}
// ##############################################################################################
// vecCoordMess
// ##############################################################################################

void pix_colortrack :: vecCoordMess(int argc, t_atom *argv){
  mode = (int)atom_getfloat(&argv[0]);
  xpix = (int)atom_getfloat(&argv[1]);
  ypix = (int)atom_getfloat(&argv[2]);
  post("test: %d %d %d", mode, xpix, ypix);
  setPixModified();}
// ##############################################################################################
// static member function
// ##############################################################################################

void pix_colortrack :: obj_setupCallback(t_class *classPtr){
	class_addmethod(classPtr, (t_method)&pix_colortrack::vecCoordMessCallback, gensym("coord"), A_GIMME, A_NULL);
    class_addmethod(classPtr, (t_method)&pix_colortrack::vecredplaneMessCallback, gensym("redplane"), A_GIMME, A_NULL);
	class_addmethod(classPtr, (t_method)&pix_colortrack::vecbluplaneMessCallback, gensym("bluplane"), A_GIMME, A_NULL);
	class_addmethod(classPtr, (t_method)&pix_colortrack::vecBoundsMessCallback, gensym("bounds"),A_GIMME,A_NULL);
}
void pix_colortrack :: vecCoordMessCallback(void *data, t_symbol *, int argc, t_atom *argv){
    GetMyClass(data)->vecCoordMess(argc, argv);}
void pix_colortrack :: vecredplaneMessCallback(void *data, t_symbol *, int argc, t_atom *argv){
    GetMyClass(data)->vecredplaneMess(argc, argv);}
void pix_colortrack :: vecbluplaneMessCallback(void *data, t_symbol *, int argc, t_atom *argv){
    GetMyClass(data)->vecbluplaneMess(argc, argv);}
void pix_colortrack :: vecBoundsMessCallback(void *data, t_symbol *, int argc, t_atom *argv){
	GetMyClass(data)->vecBoundsMess(argc, argv);}

