/*-----------------------------------------------------------------
LOG
    GEM - Graphics Environment for Multimedia

    Create a pix_colortrack with a given RGB(A)

    Copyright (c) 1997-2000 Mark Danks. mark@danks.org
    Copyright (c) Gnther Geiger. geiger@epy.co.at
    Copyright (c) 2001-2002 IOhannes m zmoelnig. forum::fr::umlute. IEM. zmoelnig@iem.kug.ac.at
    For information on usage and redistribution, and for a DISCLAIMER OF ALL
    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.

-----------------------------------------------------------------*/

/////////////////////////////////////////////////////////
//
//  pix_colortrack
//
//  crca/ucsd
//  jaime oliver 
//  mailto:joliverl@ucsd.edu
//
/////////////////////////////////////////////////////////

#ifndef INCLUDE_PIX_RECTANGLE_H_
#define INCLUDE_PIX_RECTANGLE_H_

#include "Base/GemPixObj.h"

/*-----------------------------------------------------------------
-------------------------------------------------------------------
CLASS
    pix_colortrack
    
    create a pix-colortrack with given RGB(A) 

KEYWORDS
    pix
    
DESCRIPTION

    Inlet for a list - "coord"
    Inlet for a list - "color"

    "coord" - The coordinate-vector to get RGB values of a pixel
	"redplanemess"  - the red offset and green and blue points that describe the plane to test to find the red pixels.
	"bluplanemess"  - the blue offset and green and red points that describe the plane to test to find the blue pixels.
	"boundsmess"  - list of coordinates to specify the region to be analyzed within the image.
   
-----------------------------------------------------------------*/
class GEM_EXTERN pix_colortrack : public GemPixObj
{
    CPPEXTERN_HEADER(pix_colortrack, GemPixObj)

    public:

	    //////////
	    // Constructor
    	pix_colortrack();
    	
    protected:
    	
    	//////////
    	// Destructor
    	virtual ~pix_colortrack();

	  	//////////
    	// Do the processing
    	virtual void 	processRGBAImage(imageStruct &image);
    	virtual void 	processGrayImage(imageStruct &image);
		virtual void 	processYUVImage (imageStruct &image);
   
    	//////////    
    	void	    	vecCoordMess(int argc, t_atom *argv);     	// Set the new coordinates	
    	void	    	vecredplaneMess(int argc, t_atom *argv);	// Set the red plane coordinates
    	void	    	vecbluplaneMess(int argc, t_atom *argv);	// Set the blue plane coordinates
		void	    	vecBoundsMess(int argc, t_atom *argv);		// Set xy coord for processing

    	//////////
    	// The new Color
	unsigned char	m_color[4];

	//////////
    	// The two corners
	int				m_lower_left[2];
	//////////
	int				m_upper_right[2];
	
	t_outlet *outlet1;
	t_outlet *outlet2;
	t_outlet *outlet3;
	int mode, xpix, ypix, histhresh;	
	float rr, rg, rb, br, bg, bb;
    int top, bottom, left, right;
        
    private:    
    	//////////
    	// Static member functions
		static void 	vecCoordMessCallback(void *data, t_symbol *, int argc, t_atom *argv);
    	static void 	vecredplaneMessCallback(void *data, t_symbol *, int argc, t_atom *argv);
    	static void 	vecbluplaneMessCallback(void *data, t_symbol *, int argc, t_atom *argv);
		static void 	vecBoundsMessCallback(void *data, t_symbol *, int argc, t_atom *argv);

};

#endif	// for header file
